@@ -1,23 +1,25 @@
 //! Command processor
 
 use log::debug;
-use std::cell::RefCell;
 use std::fmt;
-use std::rc::Rc;
 use unicode_segmentation::UnicodeSegmentation;
 use unicode_width::UnicodeWidthChar;
 
 use super::{Context, Helper, Result};
+use crate::error::ReadlineError;
 use crate::highlight::Highlighter;
 use crate::hint::Hint;
 use crate::history::SearchDirection;
 use crate::keymap::{Anchor, At, CharSearch, Cmd, Movement, RepeatCount, Word};
 use crate::keymap::{InputState, Invoke, Refresher};
 use crate::layout::{Layout, Position};
-use crate::line_buffer::{LineBuffer, WordAction, MAX_LINE};
+use crate::line_buffer::{
+    ChangeListener, DeleteListener, Direction, LineBuffer, NoListener, WordAction, MAX_LINE,
+};
 use crate::tty::{Renderer, Term, Terminal};
 use crate::undo::Changeset;
 use crate::validate::{ValidationContext, ValidationResult};
+use crate::KillRing;
 
 /// Represent the state during line editing.
 /// Implement rendering.
@@ -29,11 +31,12 @@ pub struct State<'out, 'prompt, H: Helper> {
     pub layout: Layout,
     saved_line_for_history: LineBuffer, // Current edited line before history browsing
     byte_buffer: [u8; 4],
-    pub changes: Rc<RefCell<Changeset>>, // changes to line, for undo/redo
+    pub changes: Changeset, // changes to line, for undo/redo
     pub helper: Option<&'out H>,
     pub ctx: Context<'out>,          // Give access to history for `hinter`
     pub hint: Option<Box<dyn Hint>>, // last hint displayed
-    highlight_char: bool,            // `true` if a char has been highlighted
+    pub highlight_char: bool,        // `true` if a char has been highlighted
+    pub forced_refresh: bool,        // `true` if line is redraw without hint or highlight_char
 }
 
 enum Info<'m> {
@@ -58,11 +61,12 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             layout: Layout::default(),
             saved_line_for_history: LineBuffer::with_capacity(MAX_LINE).can_growth(true),
             byte_buffer: [0; 4],
-            changes: Rc::new(RefCell::new(Changeset::new())),
+            changes: Changeset::new(),
             helper,
             ctx,
             hint: None,
             highlight_char: false,
+            forced_refresh: false,
         }
     }
 
@@ -79,19 +83,27 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         input_state: &mut InputState,
         rdr: &mut <Terminal as Term>::Reader,
         single_esc_abort: bool,
+        ignore_external_print: bool,
     ) -> Result<Cmd> {
         loop {
-            let rc = input_state.next_cmd(rdr, self, single_esc_abort);
-            if rc.is_err() && self.out.sigwinch() {
+            let rc = input_state.next_cmd(rdr, self, single_esc_abort, ignore_external_print);
+            if let Err(ReadlineError::WindowResized) = rc {
+                debug!(target: "rustyline", "SIGWINCH");
+                let old_cols = self.out.get_columns();
                 self.out.update_size();
-                self.prompt_size = self
-                    .out
-                    .calculate_position(self.prompt, Position::default());
-                self.refresh_line()?;
+                let new_cols = self.out.get_columns();
+                if new_cols != old_cols
+                    && (self.layout.end.row > 0 || self.layout.end.col >= new_cols)
+                {
+                    self.prompt_size = self
+                        .out
+                        .calculate_position(self.prompt, Position::default());
+                    self.refresh_line()?;
+                }
                 continue;
             }
             if let Ok(Cmd::Replace(..)) = rc {
-                self.changes.borrow_mut().begin();
+                self.changes.begin();
             }
             return rc;
         }
@@ -99,13 +111,14 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     pub fn backup(&mut self) {
         self.saved_line_for_history
-            .update(self.line.as_str(), self.line.pos());
+            .update(self.line.as_str(), self.line.pos(), &mut NoListener);
     }
 
     pub fn restore(&mut self) {
         self.line.update(
             self.saved_line_for_history.as_str(),
             self.saved_line_for_history.pos(),
+            &mut self.changes,
         );
     }
 
@@ -194,7 +207,8 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     fn highlight_char(&mut self) -> bool {
         if let Some(highlighter) = self.highlighter() {
-            let highlight_char = highlighter.highlight_char(&self.line, self.line.pos());
+            let highlight_char =
+                highlighter.highlight_char(&self.line, self.line.pos(), self.forced_refresh);
             if highlight_char {
                 self.highlight_char = true;
                 true
@@ -216,9 +230,9 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     pub fn validate(&mut self) -> Result<ValidationResult> {
         if let Some(validator) = self.helper {
-            self.changes.borrow_mut().begin();
+            self.changes.begin();
             let result = validator.validate(&mut ValidationContext::new(self))?;
-            let corrected = self.changes.borrow_mut().end();
+            let corrected = self.changes.end();
             match result {
                 ValidationResult::Incomplete => {}
                 ValidationResult::Valid(ref msg) => {
@@ -271,15 +285,15 @@ impl<'out, 'prompt, H: Helper> Refresher for State<'out, 'prompt, H> {
     }
 
     fn doing_insert(&mut self) {
-        self.changes.borrow_mut().begin();
+        self.changes.begin();
     }
 
     fn done_inserting(&mut self) {
-        self.changes.borrow_mut().end();
+        self.changes.end();
     }
 
     fn last_insert(&self) -> Option<String> {
-        self.changes.borrow().last_insert()
+        self.changes.last_insert()
     }
 
     fn is_cursor_at_end(&self) -> bool {
@@ -301,6 +315,17 @@ impl<'out, 'prompt, H: Helper> Refresher for State<'out, 'prompt, H> {
     fn pos(&self) -> usize {
         self.line.pos()
     }
+
+    fn external_print(&mut self, msg: String) -> Result<()> {
+        self.out.clear_rows(&self.layout)?;
+        self.layout.end.row = 0;
+        self.layout.cursor.row = 0;
+        self.out.write_and_flush(msg.as_str())?;
+        if !msg.ends_with('\n') {
+            self.out.write_and_flush("\n")?;
+        }
+        self.refresh_line()
+    }
 }
 
 impl<'out, 'prompt, H: Helper> fmt::Debug for State<'out, 'prompt, H> {
@@ -326,7 +351,7 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     /// Insert the character `ch` at cursor current position.
     pub fn edit_insert(&mut self, ch: char, n: RepeatCount) -> Result<()> {
-        if let Some(push) = self.line.insert(ch, n) {
+        if let Some(push) = self.line.insert(ch, n, &mut self.changes) {
             if push {
                 let prompt_size = self.prompt_size;
                 let no_previous_hint = self.hint.is_none();
@@ -344,7 +369,6 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
                     debug_assert!(self.layout.prompt_size <= self.layout.cursor);
                     debug_assert!(self.layout.cursor <= self.layout.end);
                     let bits = ch.encode_utf8(&mut self.byte_buffer);
-                    let bits = bits.as_bytes();
                     self.out.write_and_flush(bits)
                 } else {
                     self.refresh(self.prompt, prompt_size, true, Info::Hint)
@@ -359,16 +383,16 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     /// Replace a single (or n) character(s) under the cursor (Vi mode)
     pub fn edit_replace_char(&mut self, ch: char, n: RepeatCount) -> Result<()> {
-        self.changes.borrow_mut().begin();
-        let succeed = if let Some(chars) = self.line.delete(n) {
+        self.changes.begin();
+        let succeed = if let Some(chars) = self.line.delete(n, &mut self.changes) {
             let count = chars.graphemes(true).count();
-            self.line.insert(ch, count);
+            self.line.insert(ch, count, &mut self.changes);
             self.line.move_backward(1);
             true
         } else {
             false
         };
-        self.changes.borrow_mut().end();
+        self.changes.end();
         if succeed {
             self.refresh_line()
         } else {
@@ -382,7 +406,7 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             {
                 let text = ch.encode_utf8(&mut self.byte_buffer);
                 let start = self.line.pos();
-                self.line.replace(start..end, text);
+                self.line.replace(start..end, text, &mut self.changes);
             }
             self.refresh_line()
         } else {
@@ -401,7 +425,7 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         if let Anchor::After = anchor {
             self.line.move_forward(1);
         }
-        if self.line.yank(text, n).is_some() {
+        if self.line.yank(text, n, &mut self.changes).is_some() {
             if !input_state.is_emacs_mode() {
                 self.line.move_backward(1);
             }
@@ -413,13 +437,17 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     // Delete previously yanked text and yank/paste `text` at current position.
     pub fn edit_yank_pop(&mut self, yank_size: usize, text: &str) -> Result<()> {
-        self.changes.borrow_mut().begin();
-        let result = if self.line.yank_pop(yank_size, text).is_some() {
+        self.changes.begin();
+        let result = if self
+            .line
+            .yank_pop(yank_size, text, &mut self.changes)
+            .is_some()
+        {
             self.refresh_line()
         } else {
             Ok(())
         };
-        self.changes.borrow_mut().end();
+        self.changes.end();
         result
     }
 
@@ -477,8 +505,37 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         }
     }
 
-    pub fn edit_kill(&mut self, mvt: &Movement) -> Result<()> {
-        if self.line.kill(mvt) {
+    pub fn edit_kill(&mut self, mvt: &Movement, kill_ring: &mut KillRing) -> Result<()> {
+        struct Proxy<'p>(&'p mut Changeset, &'p mut KillRing);
+        let mut proxy = Proxy(&mut self.changes, kill_ring);
+        impl DeleteListener for Proxy<'_> {
+            fn start_killing(&mut self) {
+                self.1.start_killing();
+            }
+
+            fn delete(&mut self, idx: usize, string: &str, dir: Direction) {
+                self.0.delete(idx, string);
+                self.1.delete(idx, string, dir);
+            }
+
+            fn stop_killing(&mut self) {
+                self.1.stop_killing()
+            }
+        }
+        impl ChangeListener for Proxy<'_> {
+            fn insert_char(&mut self, idx: usize, c: char) {
+                self.0.insert_char(idx, c)
+            }
+
+            fn insert_str(&mut self, idx: usize, string: &str) {
+                self.0.insert_str(idx, string)
+            }
+
+            fn replace(&mut self, idx: usize, old: &str, new: &str) {
+                self.0.replace(idx, old, new)
+            }
+        }
+        if self.line.kill(mvt, &mut proxy) {
             self.refresh_line()
         } else {
             Ok(())
@@ -490,15 +547,15 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             return Ok(());
         }
         let cursor = self.line.pos();
-        self.line.insert_str(cursor, text);
+        self.line.insert_str(cursor, text, &mut self.changes);
         self.refresh_line()
     }
 
     /// Exchange the char before cursor with the character at cursor.
     pub fn edit_transpose_chars(&mut self) -> Result<()> {
-        self.changes.borrow_mut().begin();
-        let succeed = self.line.transpose_chars();
-        self.changes.borrow_mut().end();
+        self.changes.begin();
+        let succeed = self.line.transpose_chars(&mut self.changes);
+        self.changes.end();
         if succeed {
             self.refresh_line()
         } else {
@@ -551,9 +608,9 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
     }
 
     pub fn edit_word(&mut self, a: WordAction) -> Result<()> {
-        self.changes.borrow_mut().begin();
-        let succeed = self.line.edit_word(a);
-        self.changes.borrow_mut().end();
+        self.changes.begin();
+        let succeed = self.line.edit_word(a, &mut self.changes);
+        self.changes.end();
         if succeed {
             self.refresh_line()
         } else {
@@ -562,9 +619,9 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
     }
 
     pub fn edit_transpose_words(&mut self, n: RepeatCount) -> Result<()> {
-        self.changes.borrow_mut().begin();
-        let succeed = self.line.transpose_words(n);
-        self.changes.borrow_mut().end();
+        self.changes.begin();
+        let succeed = self.line.transpose_words(n, &mut self.changes);
+        self.changes.end();
         if succeed {
             self.refresh_line()
         } else {
@@ -589,16 +646,22 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         } else if self.ctx.history_index == 0 && prev {
             return Ok(());
         }
-        if prev {
-            self.ctx.history_index -= 1;
+        let (idx, dir) = if prev {
+            (self.ctx.history_index - 1, SearchDirection::Reverse)
         } else {
             self.ctx.history_index += 1;
-        }
-        if self.ctx.history_index < history.len() {
-            let buf = history.get(self.ctx.history_index).unwrap();
-            self.changes.borrow_mut().begin();
-            self.line.update(buf, buf.len());
-            self.changes.borrow_mut().end();
+            (self.ctx.history_index, SearchDirection::Forward)
+        };
+        if idx < history.len() {
+            if let Some(r) = history.get(idx, dir)? {
+                let buf = r.entry;
+                self.ctx.history_index = r.idx;
+                self.changes.begin();
+                self.line.update(&buf, buf.len(), &mut self.changes);
+                self.changes.end();
+            } else {
+                return Ok(());
+            }
         } else {
             // Restore current edited line
             self.restore();
@@ -626,11 +689,11 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             &self.line.as_str()[..self.line.pos()],
             self.ctx.history_index,
             dir,
-        ) {
+        )? {
             self.ctx.history_index = sr.idx;
-            self.changes.borrow_mut().begin();
-            self.line.update(sr.entry, sr.pos);
-            self.changes.borrow_mut().end();
+            self.changes.begin();
+            self.line.update(&sr.entry, sr.pos, &mut self.changes);
+            self.changes.end();
             self.refresh_line()
         } else {
             self.out.beep()
@@ -654,11 +717,15 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             return Ok(());
         }
         if first {
-            self.ctx.history_index = 0;
-            let buf = history.get(self.ctx.history_index).unwrap();
-            self.changes.borrow_mut().begin();
-            self.line.update(buf, buf.len());
-            self.changes.borrow_mut().end();
+            if let Some(r) = history.get(0, SearchDirection::Forward)? {
+                let buf = r.entry;
+                self.ctx.history_index = r.idx;
+                self.changes.begin();
+                self.line.update(&buf, buf.len(), &mut self.changes);
+                self.changes.end();
+            } else {
+                return Ok(());
+            }
         } else {
             self.ctx.history_index = history.len();
             // Restore current edited line
@@ -669,7 +736,7 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
 
     /// Change the indentation of the lines covered by movement
     pub fn edit_indent(&mut self, mvt: &Movement, amount: usize, dedent: bool) -> Result<()> {
-        if self.line.indent(mvt, amount, dedent) {
+        if self.line.indent(mvt, amount, dedent, &mut self.changes) {
             self.refresh_line()
         } else {
             Ok(())
@@ -683,36 +750,37 @@ pub fn init_state<'out, H: Helper>(
     line: &str,
     pos: usize,
     helper: Option<&'out H>,
-    history: &'out crate::history::History,
+    history: &'out crate::history::DefaultHistory,
 ) -> State<'out, 'static, H> {
     State {
         out,
         prompt: "",
         prompt_size: Position::default(),
-        line: LineBuffer::init(line, pos, None),
+        line: LineBuffer::init(line, pos),
         layout: Layout::default(),
         saved_line_for_history: LineBuffer::with_capacity(100),
         byte_buffer: [0; 4],
-        changes: Rc::new(RefCell::new(Changeset::new())),
+        changes: Changeset::new(),
         helper,
         ctx: Context::new(history),
         hint: Some(Box::new("hint".to_owned())),
         highlight_char: false,
+        forced_refresh: false,
     }
 }
 
 #[cfg(test)]
 mod test {
     use super::init_state;
-    use crate::history::History;
+    use crate::history::{DefaultHistory, History};
     use crate::tty::Sink;
 
     #[test]
     fn edit_history_next() {
-        let mut out = Sink::new();
-        let mut history = History::new();
-        history.add("line0");
-        history.add("line1");
+        let mut out = Sink::default();
+        let mut history = DefaultHistory::new();
+        history.add("line0").unwrap();
+        history.add("line1").unwrap();
         let line = "current edited line";
         let helper: Option<()> = None;
         let mut s = init_state(&mut out, line, 6, helper.as_ref(), &history);
